 
function WaveSlider()
{
	return Slider().setValues({
		"gxIgnoreTrackClicks" : true,
		"gxEndGap" : 4,  
		"gxIgnoreScrollWheel" : true,
		"gxKeystrokeStepSize" : 0.125,
	})
	.className("Farrago.AXSliderView")
	.trackPainter(null)
	.setAXFormatter(PercentageFormatter())	
	.knobPainter(Painter().fill(Colors.warning).size(20,20));
}

function ClipView()
{
	return ZStack([

		LayerView([
			// Full Waveform
			WaveLayer(true, MainThemeColor(Colors.tileTintPale, Colors.tileTint.brightness(0.35).darkMode), Colors.tileTint, 6),

			// Selection Waveform with mask
			WaveLayer(true, Colors.tileTint,
				MainThemeColor(Colors.tileTintPale, Colors.tileTint.brightness(0.35).darkMode), 6)
				.viewName("selectionLayer"),
			// Frame
			Layer().fillFrame().backPainter(Painter().radius(6).stroke(2, Colors.tileTint))
		]).fillFrame(),

		// selection start slider
		WaveSlider()
			.className("FGClipTimeSelectionSlider")
			.setAXTitle("Clip selection start")
			.setAXRole("Clip selection range start")
			.setAXHelp("Accessiblity press to play the selection")
			.knobPainter(Painter().fill(Colors.invisible).size(1, 1))
			.setValue(0.01, "gxKeystrokeStepSize")
			.viewName("selectStartPosition").keyPath("axSelectStartPosition").fillFrame(),

		// selection end slider
		WaveSlider()
			.className("FGClipTimeSelectionSlider")
			.setAXTitle("Clip selection end")
			.setAXRole("Clip selection range end")
			.setAXHelp("Accessiblity press to play the selection")
			.knobPainter(Painter().fill(Colors.invisible).size(1, 1))
			.setValue(0.01, "gxKeystrokeStepSize")
			.viewName("selectEndPosition")
			.keyPath("axSelectEndPosition").fillFrame(),

		// WaveView().fillFrame().setValue(false, "axIsIgnored"),
	])	
		.makeView("FGClipView")
		.keyPath("")
		.setAXTitle("Audio Clip")   
}

function WaveView() 
{ 
	return LayerView([ 
		WaveLayer(false, Colors.tileTint.withAlpha(0.2), Colors.tileTint, 6).viewName("backLayer"), 
		Layer()
			.viewName("frameLayer")
			.keyPath("")
			.setValues({
				gxNeedsDisplayOnResize: false,
				gxSetLayerContentImage: true,
				gxContentsCenter: "0.5, 0.5, 0, 0",     // resize by stretching center  pixel h and v
				"gxContentsGravity": "resize",  // resize content
			})
			.backPainter(Painter().size(40).radius(3).stroke(2, Colors.tileTintLight))
			.fillFrame(),    
	])   
	.keyPath("")
	.viewName("waveform") 
}  

function WaveLayer(fore, backColor = Colors.tileTintPale, shapedWaveColor = Colors.tileTint, radius = 6, keyPath="shapedWaveData")
{
	return Layer().setValues({ 
		gxLayout: [`H:|-0-[self]-0-|`, `V:|-0-[self]-0-|`],
		gxValueKeyPath: keyPath, // receive up	date for tint color changes 
		gxNeedsDisplayOnResize: false,
		gxHideOnNilValue: true, 
		gxContentsCenter: "0.5, 0.5, 0, 0",     // resize by stretching center  pixel h and v
		"gxContentsGravity": "resize",  // resize content
		gxBackPainter: {
			nsClass: "GXRoundRectPainter",
			gxCornerRadius: radius,
			gxFillColor: backColor, 
		},
		"gxSublayers":
		{ 
			[`shapedWave${fore ? "Fore" : "Back"}`]: {
				nsClass: "GXWaveLayer",
				trimWaveform: true,
				gxLayout: [
					`H:|-0-[self]-0-|`,
					"V:|-0-[self]-0-|"
				],
				gxLayerZPosition: 1,
				gxValueKeyPath: keyPath,
				gxHideOnNilValue: true, 
				gxNeedsDisplayOnResize: false,
				// gxShowOnKey: "waveformEnabled",
				gxBackPainter: {
					nsClass: "GXWavePainter",
					gxFillColor:  shapedWaveColor,
				},
			},
		},
	})
} 