function ToolbarButtonInternal(icon, title, titlePath, buttonPath = null, menuItems = [], action, tint = Colors.toolbarButton)
{
	return ZStack([ 
		// 
		Button() 
			.keyPath(menuItems.length ? null : buttonPath)
			.backPainter(null)
			.forePainter(null) 
			.setAXTitle(title)
			.setValue(menuItems.length != 0,"axIsIgnored") 
			.fillFrame(),
		 
		VStack([
			Button()
				.viewName("button")
				.backPainter(ImagePainter(icon).tintColor(tint.showPressed()))
				.ignoreAX() 
				.setValue(menuItems, Array.isArray(menuItems) ? "gxMenuItems" : "gxMenuItemsKeyPath")
				.keyPath(buttonPath) 
				.size(28)
				.centerH()
				.setValue(titlePath != "" ? titlePath : "", "gxToolTipKeyPath"),
			
			Spacer(),
			// Label
			Label(title) 
				.ignoreAX()
				.fontSize(10) 
				.bottomPad(0)
				.fontColor(tint)
				.keyPath(titlePath)
				.setValue(titlePath != "" ? titlePath : "", "gxToolTipKeyPath")
				.lineBreakMode(0)
				.viewName("label")
				.centerH()
		])
	])
	.ignoreAX() 
	.makeView("FGToolbarButtonView")  
	.keyPath(buttonPath) 
	.disableKey(null) 
	.toolTip(title);
}

function ToolbarButtonDark(icon, title, titlePath, buttonPath = null, menuItems = [], action, tint = Colors.toolbarButtonDark)
{
	return ToolbarButton(icon, title, titlePath, buttonPath, menuItems, action, tint)
}

function ToolbarButton(icon, title, titlePath, buttonPath = null, menuItems = [], action, tint = Colors.toolbarButton)
{
	return HStack([
		Padding(5),
		ToolbarButtonInternal(icon, title, titlePath, buttonPath, menuItems, action, tint).fillHeight(),
		Padding(5),
	])
	.makeView()   
	.height(Constants.toolbarButtonHeight)
	.backPainter(Painter().radius(8).addContrast().insets(1,0,1,0))
} 

function TransportButton(icon, tint)
{
	return Button()
		.backPainter(ImagePainter(icon).insets(0,15,0,5).align(0.5,0).offset(0,-2).tintColor(tint.showPressed()))
		.forePainter(Painter().fill(Colors.invisible));
}

function VLabelButton(icon, label, keyPath, tint = Colors.tileTint)
{
    return ToolbarButton(icon, label, "", keyPath, [], null, tint).width(40)
}

let fillStart = Colors.controlFill  
		.addStyle("on", Colors.tileTint) 
		.addStyle("pressed", Colors.tileTint.brightness(0.9).addStyle("locked", Colors.tertiaryContent)); 

function PushButtonBack()
{ 
	return CocoaButtonBack().fill(fillStart);
}

function PushButton(title, axTitle, axHelp, shadow = true, forePainter)
{  
	let titleColor = MainThemeColor(Colors.primaryContent
		.addStyle("on", Colors.primaryContent.inverted())
		.addStyle("pressed", Colors.primaryContent.inverted()),  
 		Colors.white
			.addStyle("disabled", Colors.tertiaryContent.darkMode)
		 	.addStyle("on", Colors.white.addStyle("8", Colors.black))
			.addStyle("pressed", Colors.white.addStyle("8", Colors.black)))
	
 
	function MakeButton()
	{
		return Button(title)
					.height(29)  
					.backPainter(PushButtonBack().stroke(shadow ? 0.5 : 0, shadow ? Colors.black.withAlpha(0.3) : Colors.invisible).addContrast())
					.forePainter(forePainter ? forePainter : LabelPainter(title).color(titleColor).fontSize(13))
					.setAXTitle(axTitle)
					.setAXHelp(axHelp)
	}

	if (!shadow)
	{
		return MakeButton()
	}

	return ZStack([
		// Make view to host background shadow
		View()
			.keyPath("")
			.fillFrame(), 
		MakeButton()  
			.fillWidth()
	])
	.makeView() 
	.backPainter(Painter().fill(Colors.invisible))
	.forePainter(Painter().fill(Colors.invisible));
}

function SmallPushButton(title, axTitle, axHelp, shadow = true, forePainter)
{  
	let titleColor = MainThemeColor(Colors.primaryContent
		.addStyle("on", Colors.primaryContent.inverted())
		.addStyle("pressed", Colors.primaryContent.inverted()),  
 		Colors.white
			.addStyle("disabled", Colors.tertiaryContent.darkMode)
		 	.addStyle("on", Colors.white.addStyle("8", Colors.black))
			.addStyle("pressed", Colors.white.addStyle("8", Colors.black)))
		
		
	function MakeButton()
	{
		return Button(title)
					.height(27)  
					.backPainter(PushButtonBack().stroke(shadow ? 0.5 : 0, shadow ? Colors.black.withAlpha(0.3) : Colors.invisible).addContrast())
					.forePainter(forePainter ? forePainter : LabelPainter(title).color(titleColor).fontSize(11))
					.setAXTitle(axTitle)
					.setAXHelp(axHelp)
	}

	if (!shadow)
	{
		return MakeButton()
	}

	return ZStack([
		// Make view to host background shadow
		View()
			.keyPath("")
			.fillFrame(), 
		MakeButton()  
			.fillWidth()
	])
	.makeView() 
	.backPainter(Painter().fill(Colors.invisible))
	.forePainter(Painter().fill(Colors.invisible));
}

function RadioButton()
{
	function RadioButtonBackPainter()
	{
		return Painter()
			.radius(20)
			.inset(1)
			.fill(Colors.tertiaryContent.showPressed().addStyle("on", Colors.tileTint))
	}
	
	function RadioButtonForePainter()
	{
		return Painter().radius(20).inset(5).fill(Colors.white)
	}
	
	return Button() 
		.backPainter(RadioButtonBackPainter())
		.forePainter(RadioButtonForePainter())
		.checkBoxRole()
		.size(15);
} 

function BigButton(icon, title, buttonPath = "", keyEquivalent = "", onTip="", offTip="")
{
	return ZStack([
		 
		// This button is for AX only
		Button()
			.fillFrame()
			.backPainter(null)
			.toolTip(onTip, offTip)
			.checkBoxRole()
			.keyPath(buttonPath)
			.viewName("axButton")
			,

		// Off View
		ZStack([ 
			// back
			LayerView([
				Layer()
					.viewName("buttonBack1")
					.mask(Layer("CAShapeLayer"))
					.setValue(true,"gxMaskToBounds")
					.backPainter(Painter().inset(1).fill(Colors.tileTint.withAlpha(0.5).showPressed()).radius(8).addContrast())
					.fillFrame()
					.keyPath(""),
				Layer()
					.viewName("buttonBack2")
					.mask(Layer("CAShapeLayer"))
					.setValue(true,"gxMaskToBounds")
					.backPainter(Painter().inset(1).fill(Colors.tileTint.showPressed()).radius(8).addContrast())
					.fillFrame()
					.keyPath(""), 
				Layer()
					.viewName("buttonBack3")   
					.backPainter(Painter().inset(1).fill(Colors.tileTint.showPressed()).radius(8).addContrast())
					.fillFrame()
					.keyPath(""),
			])
				.showKey(buttonPath)
				.fillFrame() 
				.centerH()
				.keyPath("")
				.ignoreAX(),

			Label(title)
				.showKey(buttonPath) 
				.fontSize(10)
				.fontColor(Colors.tileTintContent)
				.lineBreakMode(0)
				.centerH()
				.bottomPad(4)
				.keyPath("")
				.ignoreAX(),
				
			Button() 
				.viewName("buttonBack")
				.showKey(buttonPath) 
				.backPainter(null)
				.keyPath(buttonPath)
				.setAXTitle(title)
				.forePainter(
					TemplatePainter(`ImagePainter.${icon}`)
						.tintColor(Colors.tileTintContent)
						.offset(0, -6)
				)
				.fillFrame()
				.ignoreAX()
				.centerH(), 
		]),
		
		// On View
		ZStack([ 
			// back
			View()
				.fillFrame()
				.ignoreAX()
				.centerH(),
			
			Label(title)
				.hideKey(buttonPath) 
				.bottomPad(4)
				.fontSize(10)
				.fontColor(Colors.tileTint.showPressed())
				.keyPath("")
				.lineBreakMode(0)
				.centerH() 
				.ignoreAX(), 
			
			Button() 
				.viewName("buttonFore")
				.hideKey(buttonPath) 
				.backPainter(null) 
				.keyPath(buttonPath)
				.setAXTitle(title)
				.forePainter(ImagePainter(icon).tintColor(Colors.tileTint.showPressed()).offset(0, -6))
				.fillFrame()
				.ignoreAX()
				.centerH(),  
		]),  

		
	])
	.makeView("FGBigButtonView")
	.height(46)
	.width(46)
	.keyPath("")
} 

function TabButton(buttons = [])
{
	var views = [] 
	var viewNames = []
	var inset = 0
	for (t in buttons)
	{
		var isLeft = t == 0;
		var isRight = t == (buttons.length - 1);
		
		var rL = Constants.buttonRadius;
		var rR = Constants.buttonRadius;
		var iL = 0;
		var iR = 0;

		if (isLeft) { rR = 0; }
		else if (isRight) { rL = 0; }
		else { rL = 0; rR = 0; }
		
		var stackView = ZStack([  
			
			PushButton("", null, null, shadow=false)   
				.setValues(buttons[t]) 
				.height(21)
				.checkBoxRole() 
				.painterByMutating("gxBackPainter", (painter) => {   
					return painter
						.stroke(0,Colors.invisible)
						.shadow(Colors.invisible, 0, 0, 0)
						.insets(-4)
						.insets(iL, iR, iL, iR)
						.radius(0)
						.radii(rL,rR,rL,rR) 
						.addContrast()
						;
						
				})  
				.painterByMutating("gxForePainter", (painter) => {   
					return painter.insets(iL, iR, iL, iR).radii(rL,rR,rL,rR);
				}), 
		
			// Create a view that host the stroke around the off look
			View()
				.fillWidth(priority=200)
				.backPainter(Painter()
					.radii(rL,rR,rL,rR)
					.stroke(0.5, Colors.black.withAlpha(0.3)).addContrast()
				)
				.hideKey(buttons[t].gxValueKeyPath),
		]);

		viewNames.push(stackView.getViewName())
		views.push(stackView);  
	} 
	return ZStack([
		View()
			.fillFrame() 
			.backPainter(CocoaButtonBack(1, Colors.invisible)),
		HStack(views).spaceViews(viewNames).makeView().vPad(4).hPad(4),  
	])
	.makeView() 
}
