function SDLabel(text)
{
    return FlexibleLabel(text)  
        .fontSize(24)  
        .fontWeight(0.4)  
        .textAlign(0.5,0);
}

Colors.streamDeckIcon = Colors.tileTint.addStyle("disabledIcon", Colors.secondaryBackground);

markup.push({

    "SDPlayTileView" : ZStack([  
        
        // Waveform
        LayerView([   
            WaveLayer(null, null, Colors.tileTint.brightness(0.6))
                .fillFrame(),
        ])
        .fillFrame(),

        // Playhead slider
        Slider()
            .fillFrame()
            .endGap(-1) // Display off the end of the button
            .knobPainter(Painter().size(1,144).fill(Colors.tileTint))
            .trackPainter(Painter().size(1,144).fill(Colors.invisible.addStyle("playing", Colors.invisible.addStyle("on", Colors.tileTint.withAlpha(0.2)))))
            .keyPath("player.trimmedPosition") 
            .animateKey("playing"),

        // Central icon
        ZStack([
            Label()
                .keyPath("tileIconString")
                .fontSize(60)
                .centerH()
                .centerV()
                .fontWeight(0.4),  
            ImageView("streamDeckImage", Colors.tileTint.brightness(0.75))  
                .size(70) 
                .center(),
        ]).makeView().center(300),

        // Tile Title
        SDLabel() 
            .textAlign(0.5,0) 
            .addLayout("V:|-2-[self(<=80)]") // Don't let the title get too high or else it covers other elements
            .fontColor(Colors.tileTint) 
            .keyPath("title"),  

        Image("tile-icon.loop", Colors.tileTint)
            .showKey("playerLoopMode")
            .size(22)
            .bottomPad(12)
            .leftPad(15),

        // Remaining time readout
        Label()
            .rightPad(10)
            .bottomPad(10)
            .fontSize(20)
            .textAlign(1,0) 
            .animateKey("playing")
            .keyPath("remainingTimeString") 
            .fontColor(Colors.tileTint),

    ])
    .makeView("FGStreamDeckView")
    .backPainter(Painter().fill(Colors.tileTint.brightness(0.1)))
    .animateKey("playing") 
    .size(144),

    "SDTileInspectorView" : ZStack([  
            
        SDLabel() 
            .addLayout("V:|-0-[self(<=80)]") 
            .textAlign(0.5,0)   
            .fontColor(Colors.tileTint) 
            .keyPath("title"),  

        ZStack([
            ZStack([

                View()  
                    .center()
                    .showKey("showEnabledFore")
                    .fillFrame()
                    .backPainter(Painter().fill(Colors.tileTint).radius(20)),  
                Image("sd-icon", Colors.tileTint) 
                     .keyPath("")
                     .hideKey("raModelValue")
                     .enableKey("showEnabledIcon")
                     .size(55)
                     .center(), 
                Image("sd-icon", Colors.white.addStyle("8", Colors.black))
                     .keyPath("")
                     .showKey("showEnabledFore")
                     .size(55)
                     .center(),  
            ])
            .makeView()
            .centerH()
            .showKey("showInspectorButtonStyle")
            .height(80)
            .width(100), 
             
            Image("sd-icon", Colors.tileTint)
                .keyPath("")
                .hideKey("showInspectorButtonStyle")
                .fillFrame()
                .center(), 

            Image("sd-icon", null)
                .keyPath("")
                .showKey("showColorIcon")
                .fillFrame()
                .center(), 
        ])
        .makeView()
        .keyPath("")
        .height(88)
        .fillWidth()
        .centerH()
        .bottomPad(4),

        Slider()
            .topPad(126)
            .hPad(10)
            .keyPath("inspector.currentVolume")
            .knobPainter(null)
            .showKey("showVolume")
       
    ])
    .makeView("FGStreamDeckView")
    .backPainter(Painter().fill(Colors.tileTint.brightness(0.1)))
    .keyPath("") 
    .size(144), 

    "SDMasterVolumeView" : ZStack([   
        VStack([
            Padding(2),
            SDLabel()
                .textAlign(0.5,0)  
                .fontColor(Colors.tileTint)  
                .keyPath("controlType"),
            Spacer(),
        ]),  
         
        ImageView("controlImage", Colors.tileTint)
            .hideKey("showColorIcon")
            .size(88)
            .centerH()
            .bottomPad(10), 

        ImageView("controlImage", null)
            .showKey("showColorIcon")
            .size(88)
            .centerH()
            .bottomPad(10),  

        Slider()
            .topPad(115)
            .hPad(10)
            .keyPath("currentVolume")
            .knobPainter(null)
            .showKey("showVolume")
    ])
    .makeView("FGStreamDeckView")
    .backPainter(Painter().fill(Colors.tileTint.brightness(0.1))) // This improves the drawing quality greatly on the iPad
    .keyPath("") 
    .size(144), 

    "SDGlobalPlaybackView" : VStack([    
        Padding(8),
        SDLabel()
            .textAlign(0.5,0)    
            .fontColor(Colors.tileTint)  
            .keyPath("controlType"),
        ImageView("controlImage", Colors.streamDeckIcon).enableKey("showEnabled").size(88).centerH(), 
        Padding(10),
    ])
    .makeView("FGStreamDeckView")
    .backPainter(Painter().fill(Colors.tileTint.brightness(0.1))) // This improves the drawing quality greatly on the iPad
    .keyPath("") 
    .size(144), 

    "SDSwitchSetView" : VStack([  
        SDLabel()  
            .textAlign(0.5,0.5)
            .fontColor(Colors.white.addStyle("selected", Colors.tileTint)) 
            .keyPath("title"), 
    ]).makeView("FGStreamDeckView")
    .backPainter(Painter().fill(Colors.tileTint.brightness(0.1))) // This improves the drawing quality greatly on the iPad
    .size(144),

    "SDBringFarragoForwardView" : VStack([   
        SDLabel("Show\nFarrago").fontColor(Colors.tileTint).keyPath(""),
        Image("sd-icon.fg").size(72).centerH()
    ]).makeView("FGStreamDeckView")
    .backPainter(Painter().fill(Colors.tileTint.brightness(0.1))) // This improves the drawing quality greatly on the iPad
    .size(144)
});
